@@ -6,16 +6,16 @@ use warnings;
 use Data::Dumper;
 use CGI;
 
-our $VERSION = "1.13";
+our $VERSION = "1.14";
 
 =head1 NAME
 
-WWW::Form - Simple, extendable OO module for HTML form validation and display
+WWW::Form - Object-oriented module for HTML form input validation and display
 
 =head1 SYNOPSIS
 
 Simple and extendable module that allows developers to handle HTML form
-validation and display flexibly and consistently.
+input validation and display flexibly and consistently.
 
 =head1 DESCRIPTION
 
@@ -24,30 +24,23 @@ This module:
 =over
 
 =item * provides functionality to handle all of the various types of HTML
-form inputs this includes displaying HTML for the various form inputs)
+form inputs
 
 =item * handles populating form inputs with user entered data or progammer
 specified default values
 
-=item * provides robust validation of user entered input
+=item * provides support for validation of user entered input
 
 =item * handles presenting customizable error feedback to users
 
-=item * is easily extended, the WWW::Form module is designed to be easily
-inherited from, so you can add your own features.
+=item * should be easy to extend, the WWW::Form module is designed to be
+inherited from, so you can add your own features
+
+=item * Can be used in both mod_perl and CGI environments
 
 =back
 
-The most time consuming process (and it's not too bad) is creating the data
-structure used for instantiating a WWW::Form object.  Once you have a
-WWW::Form object almost all your work is done, as it will have enough
-information to handle just about everything.
-
-Before we get too involved in the details, let's take a look at a sample
-usage of the WWW::Form module in a typical setting.  The following example
-uses CGI instead of mod_perl, so if you're using mod_perl, certain pieces of
-the code would look a little different.  The WWW::Form module is used the same
-way in both environments (CGI or mod_perl), though.
+A sample usage:
 
     #!/usr/bin/perl
     use strict;
@@ -89,6 +82,7 @@ way in both environments (CGI or mod_perl), though.
         }
     }
 
+
     # Display the HTML web page
     print <<HTML;
     Content-Type: text/html
@@ -99,13 +93,16 @@ way in both environments (CGI or mod_perl), though.
     </head>
     <body>
     HTML
+
         # Display the HTML form content
         print $form->get_form_HTML(action => './form_test.pl');
+
     print <<HTML;
     </body>
     </html>
     HTML
 
+
     # Returns data structure suitable for passing to WWW::Form object
     # constructor, the keys will become the names of the HTML form inputs
     sub get_form_fields {
@@ -133,20 +130,7 @@ way in both environments (CGI or mod_perl), though.
         return \%fields;
     }
 
-=head2 Instantiating A WWW::Form Object
-
-As I said, instantiating a form object is the trickiest part.  The WWW::Form
-constructor takes three parameters.  The first parameter called $fieldsData,
-is a hash reference that describes how the form should be built.  $fieldsData
-should be keyed with values that are suitable for using as the value of the
-form inputs' name HTML attributes.  That is, if you call a key of your
-$fieldsData hash 'full_name', then you will have some type of form input whose
-name attribute will have the value 'full_name'. The values of the $fieldsData
-keys (i.e., $fieldsData->{$fieldName}) should also be hash references.  This
-hash reference will be used to tell the WWW::Form module about your form
-input.  All of these hash references will be structured similarly, however,
-there are a couple of variations to accommodate the various types of form
-inputs.  The basic structure is as follows:
+=head2 Creating WWW::Form Objects
 
  {
      # UI presentable value that will label the form input
@@ -161,100 +145,18 @@ inputs.  The basic structure is as follows:
      # An array ref of various validations that should be performed on the
      # user entered input
      validators => [],
-     # A hash ref that contains extra HTML attributes to add to the 
+     # A hash ref that contains extra HTML attributes to add to the
      # container.
      container_attributes => {},
      # A hint that will be displayed to the user near the control and its
      # label to guide him what to fill in that control. (optional)
      hint => 'text',
-     # A hash ref that contains extra HTML attributes to add to the 
+     # A hash ref that contains extra HTML attributes to add to the
      # container of the hint.
      hint_container_attributes => {},
  }
 
-So to create a WWW::Form object with one text box you would have the
-following data structure:
-
- my $fields = {
-     emailAddress => {
-         label        => 'Email address',
-         defaultValue => 'you@emailaddress.com',
-         type         => 'text',
-         validators   => [WWW::FieldValidator->new(
-             WWW::FieldValidator::WELL_FORMED_EMAIL,
-             'Make sure email address is well formed
-         )],
-         container_attributes => { 'class' => "green",},
-         hint => "Fill in a valid E-mail address",
-         hint_container_attributes => { 'style' => "border : double", },
-     }
- };
-
-You could then say the following to create that WWW::Form object:
-
-  my $form = WWW::Form->new($fields);
-
-Now let's talk about the second parameter.  If a form is submitted, the second
-parameter is used.  This parameter should be a hash reference of HTTP POST
-parameters. So if the previous form was submitted you would instantiate the
-WWW::Form object like so:
-
-  my $params = $r->param(); # or $q->Vars if you're using CGI
-  my $form   = WWW::Form->new($fields, $params);
-
-At this point, let me briefly discuss how to specify validators for your form
-inputs.
-
-The validators keys in the $fieldsData->{$fieldName} hash reference can be
-left empty, which means that the user entered input does not need to be
-validated at all, or it can take a comma separated list of WWW::FieldValidator
-objects.  The basic format for a WWW::FieldValidator constructor is as follows:
-
-  WWW::FieldValidator->new(
-      $validatorType,
-      $errorFeedbackIfFieldNotValid,
-      # Optional, depends on type of validator, if input is entered validation
-      # is run, if nothing is entered input is OK
-      $otherVarThatDependsOnValidatorType,
-      $isOptional
-  )
-
-The FieldValidator types are:
-
-  WWW::FieldValidator::WELL_FORMED_EMAIL
-  WWW::FieldValidator::MIN_STR_LENGTH
-  WWW::FieldValidator::MAX_STR_LENGTH
-  WWW::FieldValidator::REGEX_MATCH
-  WWW::FieldValidator::USER_DEFINED_SUB
-
-So to create a validator for a field that would make sure the input of said
-field was a minimum length, if any input was entered, you would have:
-
-  WWW::FieldValidator->new(
-      WWW::FieldValidator::MIN_STR_LENGTH,
-      'String must be at least 6 characters',
-      6, # input must be at least 6 chars
-      # input is only validated if user entered something if field left blank,
-      # it's OK
-      1 # field is optional
-  )
-
-Now for the third parameter.  The third parameter is simply an array reference
-of the keys of the $fieldsData hash, but the order of elements in the array
-ref should be the order that you want your form inputs to be displayed in.
-This array ref is used by the get_form_HTML method to return a form block that
-can be displayed in an HTML page.
-
-  # The third parameter will be used to generate an HTML form whose inputs
-  # will be in the order of their appearance in the array ref, note this is
-  # the constructor format you should use when instantiating form objects
-  my $form = WWW::Form->new(
-      $fieldsData,
-      $params,
-      ['name', 'emailAddress', 'password']
-  );
-
-=head2 How To Create All The Various Form Inputs
+=head2 Supported Form Inputs
 
 The following form input types are supported by the WWW::Form module (these
 values should be used for the 'type' key of your $fieldsData->{$fieldName}
@@ -320,7 +222,7 @@ be selected by default, if it is 0 it will not be selected by default.
 
 =head1 FUNCTION REFERENCE
 
-NOTE: For style conscious developers all public methods are available using
+NOTE: All methods are available using
 internalCapsStyle and underscore_separated_style. So 'isSubmitted' is also
 available as 'is_submitted', and 'getFieldHTMLRow' is also available as
 'get_field_HTML_row', and so on and so forth.
@@ -374,6 +276,7 @@ sub new {
     return $self;
 }
 
+
 =head2 validateFields
 
 Validates field's values input according to the validators
@@ -459,9 +362,9 @@ sub validateFields {
     # Return hash ref of valid fields
     return \%validFields;
 }
-
 *validate_fields = \&validateFields;
 
+
 =head2 getFields
 
 Returns hash ref of fields data.
@@ -475,9 +378,9 @@ sub getFields {
     my $self = shift;
     return $self->{fields};
 }
-
 *get_fields = \&getFields;
 
+
 =head2 resetFields
 
 Resets values and default values for all fields
@@ -498,9 +401,9 @@ sub resetFields {
             if ($args{include_defaults});
     }
 }
-
 *reset_fields = \&resetFields;
 
+
 =head2 getField
 
 Returns hash ref of field data that describes the form input that corresponds
@@ -517,9 +420,9 @@ sub getField {
     my $fieldName = shift;
     return $self->{fields}{$fieldName};
 }
-
 *get_field = \&getField;
 
+
 =head2 getFieldErrorFeedback
 
 Returns an array of all the error feedback (if any) for the specified
@@ -543,12 +446,12 @@ sub getFieldErrorFeedback {
         return ();
     }
 }
-
 *get_field_error_feedback = \&getFieldErrorFeedback;
 
+
 =head2 getFieldsOrder
 
-Returns array ref of field names in the order they should be displayed.
+Returns array ref of field names in the order that they will be displayed.
 
   Example:
 
@@ -559,9 +462,9 @@ sub getFieldsOrder {
     my $self = shift;
     return $self->{fieldsOrder};
 }
-
 *get_fields_order = \&getFieldsOrder;
 
+
 =head2 getFieldValue
 
 Returns the current value of the specified $fieldName.
@@ -576,9 +479,9 @@ sub getFieldValue {
     my $fieldName = shift;
     return $self->getField($fieldName)->{value};
 }
-
 *get_field_value = \&getFieldValue;
 
+
 =head2 isFieldValid
 
 Returns 1 or 0 depending on whether or not the specified field name is valid.
@@ -594,9 +497,9 @@ sub isFieldValid {
 
     return $self->getField($fieldName)->{isValid};
 }
-
 *is_field_valid = \&isFieldValid;
 
+
 =head2 getFieldValidators
 
 Returns array ref of validators for the passed field name.
@@ -610,9 +513,9 @@ sub getFieldValidators {
     my ($self, $fieldName) = @_;
     return $self->getField($fieldName)->{validators};
 }
-
 *get_field_validators = \&getFieldValidators;
 
+
 =head2 getFieldType
 
 Returns value of a field's 'type' key for the specified $fieldName.
@@ -627,9 +530,9 @@ sub getFieldType {
     my $fieldName = shift;
     return $self->getField($fieldName)->{type};
 }
-
 *get_field_type = \&getFieldType;
 
+
 =head2 getFieldLabel
 
 Returns the label associated with the specified $fieldName.
@@ -652,9 +555,9 @@ sub getFieldLabel {
         return $field->{label};
     }
 }
-
 *get_field_label = \&getFieldLabel;
 
+
 =head2 getFieldHint
 
 Returns the hint associated with the specified $fieldName or undef if it
@@ -673,9 +576,9 @@ sub getFieldHint {
 
     return $field->{hint};
 }
-
 *get_field_hint = \&getFieldHint;
 
+
 =head2 setFieldValue
 
 Sets the value of the specified $fieldName to $value.  You might use this if
@@ -699,9 +602,9 @@ sub setFieldValue {
         #warn("could not find field for field name: '$fieldName'");
     }
 }
-
 *set_field_value = \&setFieldValue;
 
+
 =head2 isValid
 
 Returns true if all form fields are valid or false otherwise.
@@ -724,9 +627,9 @@ sub isValid {
     my $self = shift;
     return $self->{isValid};
 }
-
 *is_valid = \&isValid;
 
+
 =head2 isSubmitted
 
 Returns true if the HTTP request method is POST.  If for some reason you're
@@ -758,9 +661,10 @@ sub isSubmitted {
         return 0;
     }
 }
-
 *is_submitted = \&isSubmitted;
 
+
+
 # Private method
 #
 # Populates fields hash for each field of the form
@@ -775,7 +679,7 @@ sub _setFields {
     # in the sub-classes more easily.
 
     foreach my $fieldName (keys %{$fieldsData}) {
-        # Use the supplied field value if one is given generally the supplied
+        # Use the supplied field value if one is given. Generally the supplied
         # data will be a hash of HTTP POST data
         my $fieldValue = '';
 
@@ -838,16 +742,6 @@ sub _setFields {
 
         # If any validators fail, this property will contain the error
         # feedback associated with those failing validators
-        #
-        # TODO : Added by Shlomif: Should it be a [] ?
-		# 12/28/2003 - Added by Ben Schmaus:
-		#   each field has an array of feedback, right now it's implemented
-		#   as an array () and not an array ref [].  I can't really think
-		#   of a great reason to make this an array reference internally instead
-		#   of an array.
-        # 15-Jan-2004 - Added by Shlomi Fish
-        #   Changing to [] as assigning an array here does not make much 
-        #   sense. (as discussed over IM). A hash value is always a scalar.
         $self->{fields}{$fieldName}{feedback} = [];
 
         # If the input type is a select box or a radio button then we need an
@@ -863,12 +757,6 @@ sub _setFields {
             ($fieldsData->{$fieldName}{extraAttributes} || "");
 
         # Add the hint
-		# 12/28/2003 - Added by Ben Schmaus:
-		#   Shlomi, could you please be more specific about the purpose of this
-		#   property?  It doesn't appear to be mentioned anywhere else in the
-		#   documentation.  I assume that this is some helpful text that can
-		#   be displayed if the user enters a field's input incorrectly.  Is that
-		#   right?
         #
         # 2004-Jan-04 - Added by Shlomi Fish:
         #  Ben, no. Actually it's a hint that will always be displayed below
@@ -885,14 +773,14 @@ sub _setFields {
             $self->{fields}{$fieldName}{hint} = $hint;
         }
 
-        # Add the container_attributes. These are HTML attributes that would 
+        # Add the container_attributes. These are HTML attributes that would
         # be added to the rows of this HTML row.
         if (my $attribs = $fieldsData->{$fieldName}{container_attributes})
         {
             $self->{fields}{$fieldName}{container_attributes} = $attribs;
         }
 
-        # Add the hint_container_attributes. These are HTML attributes that 
+        # Add the hint_container_attributes. These are HTML attributes that
         # would  be added to the Hint row of this HTML row.
         if (my $attribs = $fieldsData->{$fieldName}{hint_container_attributes})
         {
@@ -914,9 +802,10 @@ sub asString {
     my $self = shift;
     return Data::Dumper::Dumper($self);
 }
-
 *as_string = \&asString;
 
+
+
 #-----------------------------------------------------------------------------
 # Convenience methods for displaying HTML form data including form inputs,
 # labels, and error feedback
@@ -978,9 +867,9 @@ sub getFieldFormInputHTML {
         return $self->_getTextAreaHTML($fieldName, $attributesString);
     }
 }
-
 *get_field_form_input_HTML = \&getFieldFormInputHTML;
 
+
 =head2 getFieldHTMLRow
 
 Note: Need to make sure you can pass in attributesString param unnamed!
@@ -1018,12 +907,13 @@ sub _render_attributes {
 
     # We sort the keys to produce reproducible output on perl 5.8.1 and above
     # where the order of the hash keys is not deterministic
-    return join("", 
-            map { " $_=\"" . $self->_escapeValue($attribs->{$_}) . "\"" } 
+    return join("",
+            map { " $_=\"" . $self->_escapeValue($attribs->{$_}) . "\"" }
                 (sort {$a cmp $b} keys(%$attribs))
             );
 }
 
+
 sub getFieldHTMLRow {
     my $self = shift;
     my $fieldName = shift;
@@ -1083,9 +973,9 @@ sub getFieldHTMLRow {
 
     return $html;
 }
-
 *get_field_HTML_row = \&getFieldHTMLRow;
 
+
 =head2 getFieldFeedbackHTML
 
 Returns HTML error content for each vaildator belonging to $fieldName that
@@ -1126,9 +1016,9 @@ sub getFieldFeedbackHTML {
 
     return $feedbackHTML;
 }
-
 *get_field_feedback_HTML = \&getFieldFeedbackHTML;
 
+
 =head2 startForm
 
 Returns an opening HTML form tag.
@@ -1190,9 +1080,9 @@ sub startForm {
 
     return $html . '>';
 }
-
 *start_form = \&startForm;
 
+
 =head2 endForm
 
 Returns HTML to close form.
@@ -1206,9 +1096,9 @@ sub endForm {
     my $self = shift;
     return '</form>';
 }
-
 *end_form = \&endForm;
 
+
 =head2 getFormHTML
 
 Loops through the fieldsOrder array and builds markup for each form input
@@ -1244,7 +1134,7 @@ HTML attributes.
 is_file_upload - Optional boolean that should be true if your form contains
 a file input.
 
-hint_container_attributes - Optional HTML attributes for all the table rows 
+hint_container_attributes - Optional HTML attributes for all the table rows
 containing the hints.
 
 buttons - Use this if you want your form to have multiple submit buttons.  See
@@ -1293,9 +1183,10 @@ sub getFormHTML {
 
     return $html . $self->endForm() . "\n";
 }
-
 *get_form_HTML = \&getFormHTML;
 
+
+
 #-----------------------------------------------------------------------------
 # More private methods
 #-----------------------------------------------------------------------------
@@ -1413,12 +1304,14 @@ sub getSubmitButtonHTML {
     return $xhtml;
 }
 
+
 # We have lots of names for this method.  It used to be private, but now it's
 # public.
 *_get_submit_button_HTML = \&getSubmitButtonHTML;
 *get_submit_button_HTML = \&getSubmitButtonHTML;
 *_getSubmitButtonHTML = \&getSubmitButtonHTML;
 
+
 # Returns HTML to display a checkbox.
 sub _getCheckBoxHTML {
     my $self             = shift;
@@ -1490,13 +1383,7 @@ sub _getTextAreaHTML {
 		. $attributesString;
 
     $textarea .= ">";
-
-    # 16-Jan-2003: Added by Shlomi Fish
-    # TODO :
-    # There seems to be an HTML injection bug here.
-
     $textarea .= $self->_escapeValue($field->{value});
-
     $textarea .= "</textarea>";
 
     return $textarea;
@@ -1543,7 +1430,6 @@ sub _getSelectBoxHTML {
 sub _escapeValue {
     my $self = shift;
     my $string = shift;
-
     return CGI::escapeHTML($string);
 }
 
@@ -1555,11 +1441,15 @@ __END__
 
 WWW::FieldValidator
 
-To see a demo of WWW::Form and WWW::FieldValidator point your web browser
+To see some demos of WWW::Form and WWW::FieldValidator point your web browser
 to:
 
   http://www.benschmaus.com/cgi-bin/perl/form_test.pl
 
+or
+
+  http://benschmaus.com/cgi-bin/perl/form_test_subclass_example.pl
+
 The following modules are related to WWW::Form and WWW::FieldValidator, you
 might want to check them out.
 
@@ -1567,6 +1457,8 @@ Data::FormValidator
 
 Embperl::Form::Validate
 
+Rose::HTML::Form
+
 HTML::Form
 
 =head1 AUTHOR
@@ -1613,12 +1505,25 @@ Adds new public method: getSubmitButtonHTML.
 
 Adds support for escaping the value of HTML input 'value' attributes.
 
+January 5, 2005
+
+Adds README file to distribution.
+
+Makes some minor documentation changes.
+
 =head1 TODO
 
 Add more helpful error logging.
 
 Add functionality for generating client side validation.
 
+Give this module a better namespace?
+
+=head2 Extension Idea
+
+Write a subclass that supports a templating library like Text::MicroMason
+or Text::Template.
+
 =head1 THANKS
 
 Thanks to Shlomi Fish for suggestions and code submissions.
